
package jp.gr.java_conf.koto.io;

import java.io.*;

public class SafePrintWriter extends Writer {
    protected final Writer out;
    private final boolean autoFlush;
    private final String lineSeparator;
    private final int lineNumber = 1;
    
    public SafePrintWriter(Writer out) {
        this(out, false);
    }
    public SafePrintWriter(Writer out, boolean autoFlush) {
        super(out);
        this.out = out;
        this.autoFlush = autoFlush;
        lineSeparator = System.getProperty("line.separator", "\n");
    }
    public SafePrintWriter(OutputStream out) {
        this(out, false);
    }
    public SafePrintWriter(OutputStream out, boolean autoFlush) {
        this(new OutputStreamWriter(toBuffered(out)), autoFlush);
    }
    public SafePrintWriter(OutputStream out, String encoding) throws java.io.UnsupportedEncodingException {
        this(out, encoding, false);
    }
    public SafePrintWriter(OutputStream out, String encoding, boolean autoFlush) throws java.io.UnsupportedEncodingException {
        this(toWriter(toBuffered(out), encoding), autoFlush);
    }
    private static OutputStream toBuffered(OutputStream os) {
        if (os instanceof FileOutputStream) {
            return new BufferedOutputStream(os);
        } else {
            return os;
        }
    }
    private static Writer toWriter(OutputStream os, String encoding) throws java.io.UnsupportedEncodingException {
        if (encoding == null) {
            return new OutputStreamWriter(os);
        } else {
            return new OutputStreamWriter(os, encoding);
        }
    }
    
    public void flush() throws IOException {
        out.flush();
    }
    public void close() throws IOException {
        out.close();
    }
    
    public void write(int c) throws IOException {
        out.write(c);
        if (autoFlush) {
            if (c == '\n' || c == '\r') {
                flush();
            }
        }
    }
    public void write(char[] cbuf, int off, int len) throws IOException {
        out.write(cbuf, off, len);
        if (autoFlush) {
            for (int i = 0; i < len; i++) {
                char ch = cbuf[i + off];
                if (ch == '\n' || ch == '\r') {
                    flush();
                }
            }
        }
    }
    public void write(char buf[]) throws IOException {
        write(buf, 0, buf.length);
    }
    public void write(String s, int off, int len) throws IOException {
        out.write(s, off, len);
        if (autoFlush) {
            for (int i = 0; i < len; i++) {
                char ch = s.charAt(i + off);
                if (ch == '\n' || ch == '\r') {
                    flush();
                    return;
                }
            }
        }
    }
    public void write(String s) throws IOException {
        write(s, 0, s.length());
    }
    
    public SafePrintWriter print(boolean b) throws IOException {
        out.write(String.valueOf(b));
        return this;
    }
    public SafePrintWriter print(char c) throws IOException {
        write((int) c);
        return this;
    }
    public SafePrintWriter print(int i) throws IOException {
        out.write(String.valueOf(i));
        return this;
    }
    public SafePrintWriter print(long l) throws IOException {
        out.write(String.valueOf(l));
        return this;
    }
    public SafePrintWriter print(float f) throws IOException {
        out.write(String.valueOf(f));
        return this;
    }
    public SafePrintWriter print(double d) throws IOException {
        out.write(String.valueOf(d));
        return this;
    }
    public SafePrintWriter print(char[] s) throws IOException {
        write(s);
        return this;
    }
    public SafePrintWriter print(String s) throws IOException {
        write(s);
        return this;
    }
    public SafePrintWriter print(Object obj) throws IOException {
        write(obj.toString());
        return this;
    }
    
    public SafePrintWriter println() throws IOException {
        out.write(lineSeparator);
        if (autoFlush) {
            flush();
        }
        return this;
    }
    public SafePrintWriter println(boolean x) throws IOException {
        print(x);
        println();
        return this;
    }
    public SafePrintWriter println(char x) throws IOException {
        print(x);
        println();
        return this;
    }
    public SafePrintWriter println(int x) throws IOException {
        print(x);
        println();
        return this;
    }
    public SafePrintWriter println(long x) throws IOException {
        print(x);
        println();
        return this;
    }
    public SafePrintWriter println(float x) throws IOException {
        print(x);
        println();
        return this;
    }
    public SafePrintWriter println(double x) throws IOException {
        print(x);
        println();
        return this;
    }
    public SafePrintWriter println(char[] x) throws IOException {
        print(x);
        println();
        return this;
    }
    public SafePrintWriter println(String x) throws IOException {
        print(x);
        println();
        return this;
    }
    public SafePrintWriter println(Object x) throws IOException {
        print(x);
        println();
        return this;
    }
}


